package at.reisinger.server.helper;

import at.reisinger.Card;
import at.reisinger.Cards;
import at.reisinger.Package;
import at.reisinger.User;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Hilfsfunktionen für Jackson Json
 */
public class JsonHelper {

    /**
     * Wandelt ein Package in form einer ArrayList<Card> om ein Json um
     * @param listCards Cards die in dem Json stehen sollen
     * @param name Name des Packages
     * @return Json String des Packages
     */
    public static String packageListToJson(ArrayList<Card> listCards, String name){
        Package packageCards = new Package(new Cards(listCards),name, 5);
        return objToJson(packageCards);
    }

    /**
     * Wandelt ein Package Objekt in ein Json um
     * @param obj Objekt das in ein Json umgewandelt werden soll
     * @return Json String des Packages
     */
    public static String objToJson(Object obj){
        ObjectMapper objectMapper = new ObjectMapper();
        objectMapper = new ObjectMapper();
        String packageJson = "";
        if(obj != null) {
            try {
                packageJson += objectMapper.writeValueAsString(obj);
            } catch (JsonProcessingException e) {
                packageJson = "Error: JsonProcessingException -> " + e.getMessage();
            }
            return packageJson;
        }else{
            return null;
        }
    }

    /**
     * Json wird in ein Card Objekt umgewandelt
     * @param json Json das umgewandelt werden soll
     * @return Null bei JsonProcessingException
     */
    public static Card jsonToCard(String json){
        ObjectMapper objectMapper = new ObjectMapper();
        Card card = null;
        try {
           card = objectMapper.readValue(json, Card.class);
           if(card != null) {
               return card;
           }else{
               return null;
           }
        } catch (JsonProcessingException e) {
            e.printStackTrace();
            return null;
        }
    }



    /**
     * Json zu List
     * @param payload Json String
     * @return List aus dem json, bei einer JsonProcessingException null
     */
    public static List<String> jsonPayloadToList(String payload){
        ObjectMapper objectMapper = new ObjectMapper();
        try {
            return objectMapper.readValue(payload, new TypeReference<List<String>>(){});
        } catch (JsonProcessingException e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * Json zu Map
     * @param payload Json String
     * @return Map aus dem json, bei einer JsonProcessingException null
     */
    public static Map<String, Object> jsonPayloadToMap(String payload){
        ObjectMapper objectMapper = new ObjectMapper();
        try {
            return objectMapper.readValue(payload, new TypeReference<Map<String,Object>>(){});
        } catch (JsonProcessingException e) {
            e.printStackTrace();
            return null;
        }
    }

    @Deprecated
    public static String userToJson(User user){
        return objToJson(user);
    }

}
